unit ch_cplay;

interface

uses
	ch_def, ch_gen;

procedure MakeComputerMove(game : gamet; col : colortype; var src, dest : position);

implementation

const
	WorthPawn     = $100;
	WorthKnight   = $300;
	WorthBishop   = $300;
	WorthRook	  = $4c0;
	WorthQueen	  = $900;
	WorthKing	  = $1000;

	ProtectedPawn		= 3;
	ProtectPawn			= 3;
	NoFriendlyPawns		= 20;

	strength : array [None..BPok] of integer =
		( 0,
		  WorthKing, WorthQueen, WorthBishop, WorthKnight, WorthRook, WorthPawn, 0,
		  WorthKing, WorthQueen, WorthBishop, WorthKnight, WorthRook, WorthPawn, 0 );

var
	mycolor	 	: colortype;




function GetPawnStructureValue(game : gamet; col : colortype) : longint;
var
  lp, pp 	: integer;
  p1, p2, p	: position;
  pi		: piece;
  n			: longint;
  updownb	: integer;

 procedure IsProtected;
 begin
	p1.y := p.y + updownb;
	p2.y := p.y + updownb;
	p1.x := p.x - 1;
	p2.x := p.x + 1;

	if (GetP(game, p1) = pi) or (GetP(game, p2) = pi) then
		inc(n, ProtectedPawn);
 end;

 procedure DoesProtect;
 begin
	p1.y := p.y - updownb;
	p2.y := p.y - updownb;
	p1.x := p.x - 1;
	p2.x := p.x + 1;

	if (GetP(game, p1) = pi) or (GetP(game, p2) = pi) then
		inc(n, ProtectPawn);
 end;

 procedure FriendlyPawns;
 var
   l : integer;
 begin
	p1.x := p.x - 1;
	p2.x := p.x + 1;

	for l := 1 to 8 do
	begin
		p1.y := l;
		p2.y := l;

		if (GetP(game, p1) = pi) or (GetP(game, p2) = pi) then
			break;
	end;

	dec(n, NoFriendlyPawns);
 end;


begin
	n := 0;

	if col = PWhite then
	begin
		pi := WPawn;
		updownb := 1;
	end
	else
	begin
		pi := BPawn;
		updownb := -1;
	end;

	for lp := 1 to 8 do
	begin
		p.y := lp;

		for pp := 1 to 8 do
		begin
			p.x := pp;

			if GetP(game, p) = pi then
			begin
				IsProtected;
				DoesProtect;
				FriendlyPawns;
			end;
		end;
	end;

	GetPawnStructureValue := n;
end;


function GetStrength(game : gamet; col : colortype) : integer;
var
  n			: longint;
  lp, pp 	: integer;
  p		  	: position;
begin
	n := 0;

	for lp := 1 to 8 do
	begin
		p.y := lp;
		for pp := 1 to 8 do
		begin
			p.x := pp;

			if GetColor(game, p) = col then
				inc(n, Strength[GetP(game, p)]);
		end;
	end;

	GetStrength := n;
end;


function GetScore(game : gamet; col : colortype) : longint;
var
 n : longint;
begin
	n := GetStrength(game, col) -
				GetStrength(game, InvColor(col)) +
				GetPawnStructureValue(game, col) -
				GetPawnStructureValue(game, InvColor(col));

	GetScore := n;
end;


function GetBoardScore(game : gamet; col : colortype; depth : byte; var src, dest : position) : longint;
var
	lp, pp, ap, zp		: byte;
	p, mp				: position;
	g2					: gamet;
	n, maxscore			: longint;
	bsrc, bdest			: position;
	dum					: boolean;
begin
	if depth = 0 then
	begin
		GetBoardScore := GetScore(game, col);
		exit;
	end;

	move(game, g2, sizeof(game));

	maxscore := -2147483647;

	for lp := 1 to 8 do
	begin
		p.y := lp;

		for pp := 1 to 8 do
		begin
			p.x := pp;

			if GetColor(game, p) = col then
			begin
				for ap := 1 to 8 do
				begin
					mp.y := ap;
					for zp := 1 to 8 do
					begin
						mp.x := zp;

						if IsValidMove(g2, p, mp, false, dum) then
						begin

							move(game, g2, sizeof(game));

							{handle pawn promotion and castling}

							PutP(g2, mp, GetP(g2, p));
							PutP(g2, p, None);

							n := GetBoardScore(g2, InvColor(col), depth-1, bsrc, bdest);
							if n > maxscore then
							begin
								if depth = 2 then
								begin
									src := p;
									dest := mp;
								end;

								maxscore := n;
							end;
						end;
					end;
				end;
			end;
		end;
	end;

	GetBoardScore := maxscore;
end;


procedure MakeComputerMove(game : gamet; col : colortype; var src, dest : position);
begin
	mycolor := col;

	GetBoardScore(game, col, 2, src, dest);
end;



end.
