unit ch_gen;


interface


uses	ch_def,
		xcrt, dos,
		utils,
		mogutils, mogsend, mogfile, defs, users;


procedure SetUpGame(var game : gamet; white, black : string);
function  MakeGame(white, black : string) : word;

procedure DelHistory(var game : gamet);
function  GetGame(n : integer; var game : gamet) : boolean;
procedure PutGame(n : integer; game : gamet);

procedure SaveScr;
procedure ResetScr;

procedure MakePos(x, y : integer; var p : position);
function  GetP(game : gamet; p : position) : piece;
procedure PutP(var game : gamet; p : position; pi : piece);
function  GetColor(game : gamet; p : position) : colortype;
function  InvColor(c : colortype) : colortype;
procedure WritePiece(p : position; pi : piece);
procedure MovePiece(var game : gamet; old, new : position);

procedure DisplayInfo(game : gamet);
procedure DisplayScreen(game : gamet);
procedure DisplayNumbers;
procedure DisplayHistory(game : gamet);

function  PosTranslate(s : string; var p : position) : boolean;

function  IsValidMove(var game : gamet; old, new : position; disperrors : boolean; var IsEnPas : boolean) : boolean;
function  IsValidCastle(var game : gamet; old, new : position; DispErrors : boolean) : boolean;

function  LastMoveWasPawn(var game : gamet; src, dest : position) : boolean;
function  CheckInsufPieces(game : gamet) : boolean;
function  CheckDraw50(game : gamet) : boolean;

function  CheckCheck(game : gamet; c : colortype) : boolean;
function  CheckCheckMate(game : gamet; c : colortype) : boolean;

function  IsMyTurn(g : gamet) : boolean;
function  IsMyGame(g : gamet) : boolean;
function  MyOpponent(g : gamet) : string;

procedure TakeBack(var g : gamet);

function  MakeNewGame(color : string) : integer;

function  CheckForTooManyGames : boolean;
function  ListMyChessGames(prn : boolean) : integer;


implementation


const
	NormalStart : boardtype =
		( ( WRook, WKnight, WBishop, WQueen,WKing,  WBishop, WKnight, WRook ),
		  ( WPawn, WPawn,   WPawn,   WPawn, WPawn,  WPawn,   WPawn,   WPawn ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( BPawn, BPawn,   BPawn,   BPawn, BPawn,  BPawn,   BPawn,   BPawn ),
		  ( BRook, BKnight, BBishop, BQueen,BKing,  BBishop, BKnight, BRook ) );

	NullStart : boardtype =
		( ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ),
		  ( None,  None,    None,    None,  None,   None,    None,    None  ) );



procedure SetUpGame(var game : gamet; white, black : string);
begin
	fillchar(game, sizeof(game), 0);

	with game do
	begin
		name_w := white;
		name_b := black;

		turn := PWhite;
		state := NotStarted;

		move(NormalStart, board, sizeof(NormalStart));

		historyend := @history;
	end;
end;

function MakeGame(white, black : string) : word;
var
	n, x : word;
	g : gamet;
	fptr : file;
begin
	SetupGame(g, white, black);

	filemode := DenyNone + $2;
	n := 0;

	repeat
		inc(n);
		assign(fptr, cfg.gamedir+strs(n)+'.CHS');
		{$i-} reset(fptr, 1); {$I+}
		x := ioresult;
		if x = 0 then close(fptr);
	until x <> 0;

	OpenSharedFile(cfg.gamedir+strs(n)+'.CHS', IWrite+DenyAll);
	blockwrite(sf.f, g, sizeof(gamet));
	CloseSharedFile;

	MakeGame := n;
end;




procedure DelHistory(var game : gamet);
var
	h, p : ^historyt;
begin
	h := @game.historyend^;

	if (h = Nil) or (h^.prev = Nil) then exit;

	h := h^.prev;
	while h^.prev <> Nil do
	begin
		p := h^.next;
		freemem(p, sizeof(historyt));
		h := h^.prev;
	end;

	freemem(h^.next, sizeof(historyt));

	game.historyend := @game.history;
	fillchar(game.history, sizeof(historyt), 0);
end;




function GetGame(n : integer; var game : gamet) : boolean;
var
	p : ^historyt;
	ht : historyt;
begin
	GetGame := false;
	if n < 1 then exit;

	OpenSharedFile(cfg.gamedir+strs(n)+'.CHS', IRead+DenyNone);

	if filesize(sf.f) = 0 then
	begin
		CloseSharedFile;
		erase(sf.f);
		exit;
	end;

	DelHistory(game);

	blockread(sf.f, game, sizeof(gamet));

	fillchar(game.history, sizeof(historyt), 0);
	game.historyend := @game.history;

	while not eof(sf.f) do
	begin
		getmem(p, sizeof(historyt));

		game.historyend^.next := p;

		fillchar(p^, sizeof(historyt), 0);
		p^.prev := @game.historyend^;

		game.historyend := @p^;

		blockread(sf.f, ht, sizeof(historyt));

		with p^ do
		begin
			moven := ht.moven;
			src := ht.src;
			dest := ht.dest;
			pmove := ht.pmove;
			alge := ht.alge;
			promoted := ht.promoted;
			castled := ht.castled;
			captured := ht.captured;
		end;
	end;

	CloseSharedFile;

	GetGame := true;
end;

procedure PutGame(n : integer; game : gamet);
var
	h : ^historyt;
begin
	OpenSharedFile(cfg.gamedir+strs(n)+'.CHS', IWrite+DenyAll);

	Truncate(sf.f);

	blockwrite(sf.f, game, sizeof(gamet));
	h := game.history.next;

	while h <> Nil do
	begin
		blockwrite(sf.f, h^, sizeof(historyt));
		h := h^.next;
	end;

	CloseSharedFile;
end;


procedure SaveScr;
begin
	sx := xwherex; sy := xwherey;
	if sy < 15 then sy := 15;
	xwindow(1, 1, scrbx, scrby);
end;

procedure ResetScr;
begin
	xwindow(1, 15, scrbx, scrby);
	xgotoxy(sx, sy);
end;


procedure MakePos(x, y : integer; var p : position);
begin
	p.x := x;
	p.y := y;
end;


function GetP(game : gamet; p : position) : piece;
begin
	GetP := game.board[p.y, p.x];
end;

procedure PutP(var game : gamet; p : position; pi : piece);
begin
	game.board[p.y, p.x] := pi;
end;

function GetColor(game : gamet; p : position) : colortype;
var
	pi : piece;
begin
	pi := GetP(game, p);

	if pi = None then
		GetColor := PNone
	else if pi < BKing then
		GetColor := PWhite
	else
		GetColor := PBlack;
end;

function InvColor(c : colortype) : colortype;
begin
	case c of
	  PWhite: InvColor := PBlack;
	  PBlack: InvColor := PWhite;
	  PNone:  InvColor := PNone;
	end;
end;

procedure WritePiece(p : position; pi : piece);
begin
	if flipped then
		xgotoxy(29+((9-p.x)*3-3), 3+p.y)
	else
		xgotoxy(29+(p.x*3-3), 3+(9-p.y));

	if (not flipped and ((p.x mod 2)+(p.y mod 2) = 1)) or
	   (flipped and (((9-p.x) mod 2)+((9-p.y) mod 2) = 1)) then
	begin
		xtextbackground(white);
		slwrite(PieceName[pi]);
		xtextbackground(black);
	end
	else
		slwrite(PieceName[pi]);
end;

procedure MovePiece(var game : gamet; old, new : position);
var
	p : piece;
begin
	p := GetP(game, old);
	if p = None then exit;

	if GetP(game, new) <> None then exit;

	PutP(game, old, None);
	PutP(game, new, p);
end;

procedure DisplayInfo(game : gamet);
var
	s 		: string;

  procedure put_s;
  begin
	while length(stripcol(s)) > 15 do
		delete(s, length(s), 1);
	slwrite(s+sp(15-length(stripcol(s))));
  end;

begin
	s := cfg.defusrcol+GetAName(game.name_w);
	xgotoxy(10, 3);
	put_s;

	s := cfg.defusrcol+GetAName(game.name_b);
	xgotoxy(10, 5);
	put_s;

	if game.state < Finished then
	begin
		xgotoxy(4, 10);
		slwrite('\LGTurn:');
		if game.turn = PWhite then
			s := cfg.defusrcol+GetAName(game.name_w);
		xgotoxy(10, 10);
		put_s;
	end
	else
	begin
		if game.state in [DrawMut, StaleMate] then
		begin
			xgotoxy(4, 10);
			xwrite(sp(22));
		end
		else
		begin
			xgotoxy(4, 10);
			slwrite('\LG Win:');

			if game.winner = PWhite then
				s := cfg.defusrcol+GetAName(game.name_w);
			xgotoxy(10, 10);
			put_s;
		end;
	end;

	if game.takeback then
	begin
		xgotoxy(6, 12);
		slwrite('\YE'+blinky+'*Takeback*');
	end
	else
	begin
		xgotoxy(6, 12);
		xwrite(sp(20));
	end;

	s := '\WH'+strs(viewing);
	xgotoxy(10, 9);
	put_s;

	s := '\WH'+states[game.state];
	xgotoxy(10, 11);
	put_s;
end;


procedure DisplayScreen(game : gamet);
var
	lp, pp	: integer;
	p		: position;

begin
	for lp := 1 to 8 do
	begin
		for pp := 1 to 8 do
		begin
			p.x := pp;
			p.y := lp;
			WritePiece(p, GetP(game, p));
		end;
	end;

	for lp := 1 to 10 do
	begin
		xgotoxy(59, lp+3);
		xclreol;
	end;
end;

procedure DisplayNumbers;
var
	lp : integer;
begin
	xtextcolor(white);
	for lp := 1 to 8 do
	begin
		xgotoxy(26, 3+lp);
		if flipped then
			xwrite(strs(lp))
		else
			xwrite(strs(9-lp));
	end;

	for lp := 1 to 8 do
	begin
		xgotoxy(53, 3+lp);
		if flipped then
			xwrite(strs(lp))
		else
			xwrite(strs(9-lp));
	end;

	xgotoxy(29, 2);
	if flipped then
		xwrite('h  g  f  e  d  c  b  a')
	else
		xwrite('a  b  c  d  e  f  g  h');

	xgotoxy(29, 13);
	if flipped then
		xwrite('h  g  f  e  d  c  b  a')
	else
		xwrite('a  b  c  d  e  f  g  h');
end;

procedure DisplayHistory(game : gamet);
var
	p 		: ^historyt;
	lp, n 	: integer;
begin
	if game.moven > 20 then
	begin
		p := @game.historyend^;

		for lp := 1 to 18 do
			p := p^.prev;

		if game.turn = pwhite then
			p := p^.prev;

		n := ((game.moven+1) div 2) - 10;
	end
	else
	begin
		p := game.history.next;
		n := 0;
	end;

	if p = Nil then exit;

	for lp := 1 to 10 do
	begin
		xgotoxy(59, 3+lp);
		slwrite('\LG'+strs(lp+n)+'.');
		if lp+n < 10 then
			xwrite('  ')
		else if lp+n < 100 then
			xwrite(' ');

		if expert or (p^.src.x = 0) then
			slwrite('\WH'+p^.alge+sp(8-length(p^.alge)))
		else
		begin
			slwrite('\WH'+chr(p^.src.x+96)+strs(p^.src.y)+
					chr(p^.dest.x+96)+strs(p^.dest.y));
			if p^.promoted <> None then
				xwrite(stripcol(PieceName[p^.promoted]))
			else
				xwrite(' ');
		end;

		p := p^.next;

		if (p <> Nil) and ((lp < 10) or ((game.turn = pwhite) and (lp = 10))) then
		begin
			if expert or (p^.src.x = 0) then
			begin
				slwrite(p^.alge);
				xclreol;
			end
			else
			begin
				slwrite('   '+chr(p^.src.x+96)+strs(p^.src.y)+
						chr(p^.dest.x+96)+strs(p^.dest.y));

				if p^.promoted <> None then
					xwrite(stripcol(PieceName[p^.promoted]));
			end;
		end
		else
			xclreol;

		if p <> Nil then p := p^.next;

		if p = Nil then lp := 10;
	end;
end;



function PosTranslate(s : string; var p : position) : boolean;
begin
	s := ups(s);

	PosTranslate := false;

	if length(s) < 2 then exit;

	if not (s[1] in ['A'..'H']) then
		exit;

	if not (s[2] in ['0'..'8']) then
		exit;

	p.x := ord(s[1]) - 64;
	p.y := ord(s[2]) - 48;

	PosTranslate := true;
end;


function IsValidMove(var game : gamet; old, new : position; disperrors : boolean; var IsEnPas : boolean) : boolean;
var
	a, b, c, d, lp, pp 	: integer;
	ps, ps2				: position;
begin
	IsValidMove := false;
	IsEnPas := false;

	case GetP(game, old) of

	  WPawn:
		begin
			if (((old.y = 2) and (new.y = 4)) or
			   (new.y = old.y+1)) and (old.x = new.x) then
			begin
                if GetP(game, new) = None then
					IsValidMove := true;
				exit;
			end;

			if (new.y = old.y+1) and
			   ( (new.x = old.x-1) or (new.x = old.x+1) ) and
			   (GetColor(game, new) = PBlack) then
			begin
				PutP(game, new, None);

				IsValidMove := true;
				exit;
			end;

            ps.y := new.y-1;
			ps.x := new.x;

            ps2.y := new.y+1;
            ps2.x := new.x;

            if (old.y = 5) and (new.y = 6) and (GetP(game, ps) = BPawn) and
			   ( (new.x-1 = old.x) or (new.x+1 = old.x) ) and
			   ( LastMoveWasPawn(game, ps2, ps) = true ) then
            begin
				PutP(game, ps, None);
                WritePiece(ps, None);

				IsEnPas := true;
                IsValidMove := true;
                exit;
            end;

		end;

	  BPawn:
		begin
			if (((old.y = 7) and (new.y = 5)) or
			   (new.y = old.y-1)) and (old.x = new.x) then
			begin
				if GetP(game, new) = None then
					IsValidMove := true;
				exit;
			end;

			if (new.y = old.y-1) and
			   ( (new.x = old.x-1) or (new.x = old.x+1) ) and
			   (GetColor(game, new) = PWhite) then
			begin
				PutP(game, new, None);

				IsValidMove := true;
				exit;
			end;

			ps.y := new.y+1;
            ps.x := new.x;

            ps2.y := new.y-1;
            ps2.x := new.x;

			if (old.y = 4) and (new.y = 3) and (GetP(game, ps) = WPawn) and
               ( (new.x-1 = old.x) or (new.x+1 = old.x) ) and
			   ( LastMoveWasPawn(game, ps2, ps) = true ) then
			begin
				PutP(game, ps, None);
                WritePiece(ps, None);

				IsEnPas := true;
				IsValidMove := true;
				exit;
			end;

		end;

	  WRook, BRook:
		begin
			if (old.y = new.y) then
			begin
				if old.x < new.x then
				begin
					a := old.x;
					b := new.x;
				end
				else
				begin
					a := new.x;
					b := old.x;
				end;

				for lp := a+1 to b-1 do
				begin
					MakePos(lp, old.y, ps);
					if GetP(game, ps) <> None then
						exit;
				end;

				if GetP(game, new) <> None then
				begin
					if GetColor(game, new) = InvColor(GetColor(Game, old)) then
						PutP(game, new, None)
					else
						exit;
				end;

				IsValidMove := true;
				exit;
			end;

			if (old.x = new.x) then
			begin
				if old.y < new.y then
				begin
					a := old.y;
					b := new.y;
				end
				else
				begin
					a := new.y;
					b := old.y;
				end;

				for lp := a+1 to b-1 do
				begin
					MakePos(old.x, lp, ps);
					if GetP(game, ps) <> None then
						exit;
				end;

                if GetP(game, new) <> None then
				begin
					if GetColor(game, new) = InvColor(GetColor(Game, old)) then
						PutP(game, new, None)
					else
						exit;
				end;

				IsValidMove := true;
				exit;
			end;
		end;


		WBishop, BBishop:
        begin
			if abs(old.y-new.y) = abs(old.x-new.x) then
			begin
				if old.x < new.x then
				begin
					a := old.x;
					b := new.x;

					if old.y < new.y then
                    begin
	                	c := old.y + 1;
                        d := 1;
                    end
	                else
                    begin
						c := old.y - 1;
                        d := -1;
					end;
				end
				else
				begin
					a := new.x;
					b := old.x;

                    if old.y < new.y then
                    begin
                    	c := new.y - 1;
                        d := -1;
					end
                    else
                    begin
                    	c := new.y + 1;
                        d := 1;
                    end;
				end;


				for lp := a+1 to b-1 do
				begin
					MakePos(lp, c, ps);
					if GetP(game, ps) <> None then
						exit;

                    inc(c, d);
				end;

				if GetP(game, new) <> None then
				begin
					if GetColor(game, new) = InvColor(GetColor(Game, old)) then
						PutP(game, new, None)
					else
						exit;
				end;

				IsValidMove := true;
				exit;
			end;
		end;


	    WQueen, BQueen:
		begin
			if (old.y = new.y) then
			begin
				if old.x < new.x then
				begin
					a := old.x;
					b := new.x;
				end
				else
				begin
					a := new.x;
					b := old.x;
				end;

				for lp := a+1 to b-1 do
				begin
					MakePos(lp, old.y, ps);
					if GetP(game, ps) <> None then
						exit;
				end;

				if GetP(game, new) <> None then
				begin
					if GetColor(game, new) = InvColor(GetColor(Game, old)) then
						PutP(game, new, None)
					else
						exit;
				end;

				IsValidMove := true;
				exit;
			end;

			if (old.x = new.x) then
			begin
				if old.y < new.y then
				begin
					a := old.y;
					b := new.y;
				end
				else
				begin
					a := new.y;
					b := old.y;
				end;

				for lp := a+1 to b-1 do
				begin
					MakePos(old.x, lp, ps);
					if GetP(game, ps) <> None then
						exit;
				end;

				if GetP(game, new) <> None then
				begin
					if GetColor(game, new) = InvColor(GetColor(Game, old)) then
						PutP(game, new, None)
					else
						exit;
				end;

				IsValidMove := true;
				exit;
			end;

			if abs(old.y-new.y) = abs(old.x-new.x) then
			begin
				if old.x < new.x then
				begin
					a := old.x;
					b := new.x;

	                if old.y < new.y then
					begin
	                	c := old.y + 1;
                        d := 1;
                    end
	                else
                    begin
						c := old.y - 1;
                        d := -1;
                    end;
				end
				else
				begin
					a := new.x;
					b := old.x;

                    if old.y < new.y then
                    begin
                    	c := new.y - 1;
                        d := -1;
					end
                    else
                    begin
						c := new.y + 1;
						d := 1;
                    end;
				end;


				for lp := a+1 to b-1 do
				begin
					MakePos(lp, c, ps);
					if GetP(game, ps) <> None then
						exit;

                    inc(c, d);
				end;

				if GetP(game, new) <> None then
				begin
					if GetColor(game, new) = InvColor(GetColor(Game, old)) then
						PutP(game, new, None)
					else
						exit;
				end;

				IsValidMove := true;
				exit;
			end;
		end;

		WKnight, BKnight:
		begin
			if ( (old.x = new.x+1) and ( (old.y = new.y+2) or (old.y = new.y-2) ) ) or
			   ( (old.x = new.x-1) and ( (old.y = new.y+2) or (old.y = new.y-2) ) ) or
			   ( (old.x = new.x+2) and ( (old.y = new.y+1) or (old.y = new.y-1) ) ) or
			   ( (old.x = new.x-2) and ( (old.y = new.y+1) or (old.y = new.y-1) ) ) then
			begin
				if GetP(game, new) <> None then
				begin
					if GetColor(game, new) = InvColor(GetColor(Game, old)) then
						PutP(game, new, None)
					else
						exit;
				end;

				IsValidMove := true;
				exit;
			end;
		end;

		WKing, BKing:
		begin
			if (abs(old.y-new.y) <= 1) and (abs(old.x-new.x) <= 1) then
			begin
				if GetP(game, new) <> None then
				begin
					if GetColor(game, new) = InvColor(GetColor(Game, old)) then
						PutP(game, new, None)
					else
						exit;
				end;

				IsValidMove := true;
				exit;
			end;

			if ( (GetP(game, old) = WKing) and
				 (old.x = 5) and (old.y = 1) and (new.y = 1) ) or
			   ( (GetP(game, old) = BKing) and
				 (old.x = 5) and (old.y = 8) and (new.y = 8) ) then
			begin
				if game.state = Check then
				begin
					if DispErrors then
					begin
						ResetScr;
						slwriteln('\WH * You may not castle while in check.');
					end;
					exit;
				end;

				IsValidMove := IsValidCastle(game, old, new, DispErrors);
				exit;
			end;
		end;
	end;
end;

function IsValidCastle(var game : gamet; old, new : position; DispErrors : boolean) : boolean;
var
	h : ^historyt;
	p : piece;
	ps : position;
	g2 : gamet;
	c  : colortype;
begin
	IsValidCastle := false;

    h := @game.history;
    h := h^.next;

	p := GetP(game, old);
    if p = WKing then
    	c := PWhite
    else
    	c := PBlack;

    while h <> Nil do
    begin
        if ((h^.pmove = WRook) and (p = WKing)) or
		   ((h^.pmove = BRook) and (p = BKing)) then
		begin
			if ((new.x = 3) and (h^.src.x = 1)) or
			   ((new.x = 7) and (h^.src.x = 8)) then
			begin
				if DispErrors then
				begin
					ResetScr;
					slwriteln('\WH * Your rook has already been moved.');
				end;
				exit;
			end;
		end;

		if h^.pmove = p then
		begin
			if DispErrors then
			begin
				ResetScr;
				slwriteln('\WH * Your king has already been moved.');
			end;
			exit;
        end;

		h := h^.next;
	end;

	ps.y := new.y;
	if (new.x = 3) then
	begin
		for ps.x := 2 to 4 do
		begin
			if GetP(game, ps) <> None then
			begin
				if DispErrors then
				begin
					ResetScr;
					slwriteln('\WH * Your castling path is blocked.');
				end;
				exit;
			end;
		end;

		move(game, g2, sizeof(game));

		PutP(g2, old, None);
		dec(old.x);
		PutP(g2, old, p);
		if CheckCheck(g2, c) then
		begin
			if DispErrors then
			begin
				ResetScr;
				slwriteln('\WH * You may not castle through check.');
			end;
			exit;
		end;

		PutP(g2, old, None);
		dec(old.x);
		PutP(g2, old, p);
		if CheckCheck(g2, c) then
		begin
			if DispErrors then
			begin
				ResetScr;
				slwriteln('\WH * You may not castle into check.');
			end;
			exit;
		end;

		move(g2, game, sizeof(game));
		new.x := 1;
		PutP(game, new, None);
		if DispErrors then
			WritePiece(new, None);
		new.x := 4;
		if p = WKing then
		begin
			PutP(game, new, WRook);
			if DispErrors then
				WritePiece(new, WRook);
		end
		else
		begin
			PutP(game, new, BRook);
			if DispErrors then
				WritePiece(new, BRook);
		end;

		if DispErrors then
			WritePiece(old, p);
	end
	else
	begin
		ps.y := new.y;
		for ps.x := 6 to 7 do
		begin
			if GetP(game, ps) <> None then
			begin
				if DispErrors then
				begin
					ResetScr;
					slwriteln('\WH * Your castling path is blocked.');
				end;
				exit;
			end;
		end;

		move(game, g2, sizeof(game));

		PutP(g2, old, None);
		inc(old.x);
		PutP(g2, old, p);
		if CheckCheck(g2, c) then
		begin
			if DispErrors then
			begin
				ResetScr;
				slwriteln('\WH * You may not castle through check.');
			end;
			exit;
		end;

		PutP(g2, old, None);
		inc(old.x);
		PutP(g2, old, p);
		if CheckCheck(g2, c) then
		begin
			if DispErrors then
			begin
				ResetScr;
				slwriteln('\WH * You may not castle into check.');
			end;
			exit;
		end;

        move(g2, game, sizeof(game));
        new.x := 8;
		PutP(game, new, None);
		if DispErrors then
			WritePiece(new, None);
        new.x := 6;
        if p = WKing then
		begin
			PutP(game, new, WRook);
			if DispErrors then
				WritePiece(new, WRook);
		end
		else
		begin
			PutP(game, new, BRook);
			if DispErrors then
				WritePiece(new, BRook);
		end;

		if DispErrors then
			WritePiece(old, p);
	end;

	IsValidCastle := true;
end;


function LastMoveWasPawn(var game : gamet; src, dest : position) : boolean;
var
	p : ^historyt;
begin
	p := @game.historyend^;

	LastMoveWasPawn := (p^.src.x = src.x) and (p^.src.y = src.y) and
					   (p^.dest.x = dest.x) and (p^.dest.y = dest.y) and
					   (p^.pmove in [WPawn, BPawn]);
end;



function CheckInsufPieces(game : gamet) : boolean;
var
	lp, pp 	: integer;
	p	 	: position;
	g2		: gamet;
	lpp		: piece;

	ps 		: array [None..BPok] of integer;
begin
	for lpp := None to BPok do
		ps[lpp] := 0;

	for lp := 1 to 8 do
	begin
		p.y := lp;
		for pp := 1 to 8 do
		begin
			p.x := pp;
			inc(ps[GetP(game, p)]);
		end;
	end;

	CheckInsufPieces := false;

	if (ps[BPawn] > 0) or (ps[WPawn] > 0) or
	   (ps[BQueen] > 0) or (ps[WQueen] > 0) or
	   (ps[BRook] > 0) or (ps[WRook] > 0) or
	   (ps[BKnight] > 1) or (ps[WKnight] > 1) or
	   (ps[BBishop] > 1) or (ps[WBishop] > 1) then
		exit;

	if ((ps[BBishop] > 0) and (ps[BKnight] > 0)) or
	   ((ps[WBishop] > 0) and (ps[WKnight] > 0)) then
		exit;

	CheckInsufPieces := true;
end;


function CheckDraw50(game : gamet) : boolean;
var
  p		: ^historyt;
  lp    : integer;
begin
	CheckDraw50 := false;

	if game.moven < 100 then
		exit;

	p := @game.historyend^;

	for lp := 0 to 100 do
	begin
		if pos('x', p^.alge) > 0 then
			exit;

		if p^.pmove in [BPawn, WPawn] then
			exit;

		p := p^.prev;
	end;

	CheckDraw50 := true;
end;



function CheckCheck(game : gamet; c : colortype) : boolean;
var
	lp, pp 	: integer;
	p, kp 	: position;
	g2		: gamet;
	dum		: boolean;
begin
	kp.x := 0;
	for lp := 1 to 8 do
	begin
		p.y := lp;
		for pp := 1 to 8 do
		begin
			p.x := pp;
			if (GetP(game, p) in [BKing, WKing]) and
			   (GetColor(game, p) = c) then
			begin
				kp.x := p.x;
				kp.y := p.y;
				lp := 8;
				pp := 8;
			end;
		end;
	end;

	if kp.x = 0 then
	begin
		CheckCheck := false;
		exit;
	end;

	move(game, g2, sizeof(game));

	for lp := 1 to 8 do
	begin
		p.y := lp;
		for pp := 1 to 8 do
		begin
			p.x := pp;

			if GetColor(game, p) = InvColor(c) then
			begin
				if IsValidMove(g2, p, kp, false, dum) then
				begin
					CheckCheck := true;
					exit;
				end;

				move(game, g2, sizeof(game));
			end;
		end;
	end;

	CheckCheck := false;
end;


function CheckCheckMate(game : gamet; c : colortype) : boolean;
var
	lp, pp, ap, zp  : integer;
	p, kp, mp 	    : position;
	g2		        : gamet;
	dum				: boolean;
begin
	kp.x := 0;
	for lp := 1 to 8 do
	begin
		p.y := lp;
		for pp := 1 to 8 do
		begin
			p.x := pp;
			if (GetP(game, p) in [BKing, WKing]) and
			   (GetColor(game, p) = c) then
			begin
				kp.x := p.x;
				kp.y := p.y;
				lp := 8;
				pp := 8;
			end;
		end;
	end;

	if kp.x = 0 then
	begin
		CheckCheckMate := false;
		exit;
	end;

	move(game, g2, sizeof(game));

	for lp := 1 to 8 do
	begin
		p.y := lp;
		for pp := 1 to 8 do
		begin
			p.x := pp;

			if GetColor(game, p) = c then
			begin
				for ap := 1 to 8 do
				begin
					mp.y := ap;
                	for zp := 1 to 8 do
                    begin
                    	mp.x := zp;

						if IsValidMove(g2, p, mp, false, dum) then
						begin
                            PutP(g2, mp, GetP(g2, p));
                            PutP(g2, p, None);
							if CheckCheck(g2, c) = false then
							begin
								CheckCheckMate := false;
								exit;
							end;
                            move(game, g2, sizeof(game));
                        end;
                    end;
				end;
			end;
		end;
	end;

	CheckCheckMate := true;
end;


function IsMyTurn(g : gamet) : boolean;
begin
	IsMyTurn := ( (g.turn = PWhite) and (g.name_w = myrec.name) ) or
				( (g.turn = PBlack) and (g.name_b = myrec.name) );
end;

function IsMyGame(g : gamet) : boolean;
begin
	IsMyGame := (g.name_w = myrec.name) or (g.name_b = myrec.name);
end;

function MyOpponent(g : gamet) : string;
begin
	if g.name_w = myrec.name then
		MyOpponent := g.name_b
	else if g.name_b = myrec.name then
		MyOpponent := g.name_w
	else
		MyOpponent := '';
end;


procedure TakeBack(var g : gamet);
var
  p : ^historyt;
  n : integer;
  p1, p2 : position;
  pi : piece;
begin
	if not IsMyGame(g) then
	begin
		slwriteln('\WH * You aren''t playing in this game.');
		exit;
	end;

	if g.moven < 2 then
	begin
		slwriteln('\WH * No moves have been made yet.');
		exit;
	end;

	if g.state > Finished then
	begin
		slwriteln('\WH * This game has been finished.');
		exit;
	end;

	if not g.takeback then
	begin
		if IsMyTurn(g) then
		begin
			slwriteln('\WH * You can''t take back your opponent''s move.');
			exit;
		end;

		slwriteln('\WH * You have submitted a takeback move.');
		n := GetUserNode(GetAName(MyOpponent(g)), true);
		if n > 0 then
			SendThisMsgP('\WH * Your opponent in game \LG'+strs(viewing)+'\WH has submitted a takeback.  '+
				'Type \lc/takeback\wh to accept.'#13#10, 1, n);

		g.takeback := true;
	end
	else
	begin
		slwriteln('\WH * You have accepted your opponent''s takeback move.');

		p := @g.historyend^;

		PutP(g, p^.dest, p^.captured);
		PutP(g, p^.src,  p^.pmove);

		if p^.castled then
		begin
			if p^.dest.x = 7 then
			begin
				MakePos(6, p^.dest.y, p1);
				MakePos(8, p^.dest.y, p2);
			end
			else
			begin
				MakePos(4, p^.dest.y, p1);
				MakePos(1, p^.dest.y, p2);
			end;

			PutP(g, p2, GetP(g, p1));
			PutP(g, p1, None);
		end;

		g.historyend := g.historyend^.prev;
		g.historyend^.next := Nil;

		g.turn := InvColor(g.turn);
		dec(g.moven);

		freemem(p, sizeof(historyt));

		g.takeback := false;
	end;

	PutGame(viewing, g);

	SendThisMsgAll('CHESS-UPD '+strs(viewing), 5);

	SaveScr;
	xwindow(1, 1, scrbx, scrby);
{	DisplayScreen(g);}
	DisplayInfo(g);
	DisplayHistory(g);
	ResetScr;
end;


function MakeNewGame(color : string) : integer;
var
  n		: integer;

begin
	MakeNewGame := 0;

	if CheckForTooManyGames then exit;

	color := ups(color);
	if (color <> 'WHITE') and (color <> 'BLACK') then
	begin
		slwriteln('\WH * You must specify either \LCwhite\WH or \LCblack\WH.');
		exit;
	end;

	if color = 'WHITE' then
		n := MakeGame(myrec.name, '')
	else
		n := MakeGame('', myrec.name);

	slwriteln('\WH * Game number \LG'+strs(n)+'\WH has been created.');
	MakeNewGame := n;
end;


function CheckForTooManyGames : boolean;
var
	n : integer;
begin
	CheckForTooManyGames := true;

	n := ListMyChessGames(false);
	if ((n >= 1) and (user.access.msglevel < 100)) then
	begin
		slwriteln('\WH * Being an non-donating user, you can play one chess game at a time!');
		slwriteln('   Please read bulletin two which details donation information.');
		exit;
	end;

	if ((n >= 5) and (user.access.msglevel < 250)) then
	begin
		slwriteln('\WH * You can only play in five games at one time.');
		exit;
	end;

	CheckForTooManyGames := false;
end;


function ListMyChessGames(prn : boolean) : integer;
var
	sr		: searchrec;
	gr  	: gamet;
	s, t	: string;
	n		: integer;
	pycg 	: boolean;

begin
	findfirst(cfg.gamedir+'*.CHS', 0, sr);

	n := 0;
	pycg := false;

	while DosError = 0 do
	begin
		OpenSharedFile(cfg.gamedir+sr.name, IRead+DenyNone);
		blockread(sf.f, gr, sizeof(gr));
		CloseSharedFile;

		if gr.state < Finished then
		begin
			s := copy(sr.name, 1, pos('.', sr.name)-1);
			t := '';

			if (gr.name_w = myrec.name) then
				t := GetAName(gr.name_b);

			if (gr.name_b = myrec.name) then
				t := GetAName(gr.name_w);

			if t <> '' then
			begin
				if prn then
				begin
					if not pycg then
					begin
						pycg := true;
						slwriteln('\WH * Chess games you are involved in:');
					end;

					slwrite('\LG   Game \YE'+s+'\LG against '+cfg.defusrcol+t+'\LG.  ');

					if t = '' then
						slwriteln('Waiting for an opponent.')
					else if (gr.turn = PWhite) and (gr.name_w = myrec.name) then
						slwriteln('It is \YEyour turn\LG.')
					else
						slwriteln('It is your opponent''s turn.');
				end;
				inc(n);
			end;
		end;

		FindNext(sr);
	end;

	listmychessgames := n;
end;


end.

