(*
 *   FOSSIL.PAS - Fossil routines made for use with a communications
 *		fossil driver.
 *
 *   Adapted from FOSSIL.C on 10Sep96.
 *
 *   Copyright (C)1993-1996 LastByte Software.
 *   All rights reserved.
 *
 *)


unit fossil;

interface

uses
	crt, dos;

type _driverinfo = record
	size		: word;
	fossil_ver	: byte;
	driver_rev	: byte;
	id			: pointer;

	in_buf_size,
	in_bytes_avail,
	out_buf_size,
	out_bytes_avail		: word;

	screen_x,
	screen_y,
	baud_mask			: byte;
end;

var
	port		: word;


function  x_initialize(prt : word) : byte;
procedure x_deinitialize;
function  x_setbaud(baud : word; parity, stopbits, length : byte) : word;
function  x_putc(ch : byte) : word;
function  x_getc : byte;
function  x_status : word;
procedure x_flush_out;
procedure x_purge_out;
procedure x_purge_in;
function  x_dtr(dtr : byte) : byte;
function  x_read(ptr : pointer; size : word) : word;
function  x_write(ptr : pointer; size : word) : word;
procedure x_getdriverinfo(var driver : _driverinfo);


function xCarrier : boolean;
function xInOverFlow : boolean;
function xOutOverFlow : boolean;
function xOutEmpty : boolean;
function xInWaiting : boolean;
function xInEmpty : boolean;


procedure xstrout(s : string);
procedure Hangup;



implementation



function x_initialize(prt : word) : byte;
var
	regs	: registers;
begin
	regs.dx := prt;
	port := prt;

	intr($14, regs);

	if (regs.ax = $1954) then
		x_initialize := regs.bh
	else
		x_initialize := 0;
end;

function x_setbaud(baud : word; parity, stopbits, length : byte) : word;
var regs : registers;
begin
	with regs do begin

	x_setbaud := 0;

	case baud of
		300:   al := 64;
		600:   al := 96;
		1200:  al := 128;
		2400:  al := 160;
		4800:  al := 192;
		9600:  al := 224;
		19200: al := 0;
		38400: al := 32;
	else
		exit;
	end;

	case parity of
		0:	;
		1:	al := al or 8;
		2:  al := al or 24;
	else
		exit;
	end;

	case stopbits of
		2:	al := al or 4;
		1: 	;
	else
		exit;
	end;

	if (length < 5) or (length > 8) then exit;

	al := al + length-5;
	dx := port;
	ah := 0;

	intr($14, regs);

	x_setbaud := ax;

	end;
end;

function x_putc(ch : byte) : word;
var regs : registers;
begin
	with regs do begin

	al := ch;
	dx := port;
	ah := 1;
	intr($14, regs);

	x_putc := ax;

	end;
end;

function x_getc : byte;
var regs : registers;
begin
	with regs do begin

	dx := port;
	ah := 2;
	intr($14, regs);

	x_getc := al;

	end;
end;

function x_status : word;
var regs : registers;
begin
	with regs do begin

	dx := port;
	ah := 3;
	intr($14, regs);

	x_status := ax;

	end;
end;

procedure x_flush_out;
var regs : registers;
begin
	with regs do begin

	dx := port;
	ah := 8;
	intr($14, regs);

	end;
end;

procedure x_purge_out;
var regs : registers;
begin
	with regs do begin

	dx := port;
	ah := $09;
	intr($14, regs);

	end;
end;

procedure x_deinitialize;
var regs : registers;
begin
	with regs do begin

	dx := port;
	ah := 5;
	intr($14, regs);

	end;
end;

function x_dtr(dtr : byte) : byte;
var regs : registers;
begin
	with regs do begin

	dx := port;
	ah := 6;
	al := dtr;
	intr($14, regs);

	x_dtr := al;

	end;
end;

procedure x_purge_in;
var regs : registers;
begin
	with regs do begin

	dx := port;
	ax := $0a;
	intr($14, regs);

	end;
end;

(*

unsigned char x_putcn(unsigned char ch)
{
	if (locallogin)
		return 0;

	regs.h.ah = 0x0b;
	regs.h.al = ch;
	regs.x.dx = port;
	int86(0x14, &regs, &regs);
	return (unsigned char)regs.x.ax;
}

unsigned int x_peekc(void)
{
	if (locallogin)
		return 0;

	regs.h.ah = 0x0c;
	regs.x.dx = port;
	int86(0x14, &regs, &regs);
	return (unsigned char)regs.x.ax;
}

unsigned int k_peekc(void)
{
	regs.h.ah = 0x0d;
	regs.x.dx = port;
	int86(0x14, &regs, &regs);
	return regs.x.ax;
}

unsigned int k_getc(void)
{
	regs.h.ah = 0x0e;
	regs.x.dx = port;
	int86(0x14, &regs, &regs);
	return regs.x.ax;
}

void x_reboot(char type)
{
	regs.h.ah = 0x17;
	regs.h.al = type;
	regs.x.dx = port;
	int86(0x14, &regs, &regs);
}
*)

function x_read(ptr : pointer; size : word) : word;
var regs : registers;
begin
	with regs do begin

	cx := size;
	es := seg(ptr);
	di := ofs(ptr);
	ah := $18;
	dx := port;
	intr($14, regs);

	x_read := ax;

	end;
end;

function x_write(ptr : pointer; size : word) : word;
var regs : registers;
begin
	with regs do begin

	cx := size;
	es := seg(ptr);
	di := ofs(ptr);
	ah := $19;
	dx := port;
	intr($14, regs);

	x_write := ax;

	end;
end;


procedure x_getdriverinfo(var driver : _driverinfo);
var regs : registers;
begin
	with regs do begin

	cx := sizeof(_driverinfo);
	es := seg(driver);
	di := ofs(driver);
	dx := port;
	ah := $1b;
	intr($14, regs);

	end;
end;







function xCarrier : boolean;
begin
	xCarrier := (x_status and 128) > 0;
end;

function xInOverFlow : boolean;
begin
	xInOverFlow := (x_status and 512) > 0;
end;

function xOutOverFlow : boolean;
begin
	xOutOverFlow := (x_status and 8192) = 0;
end;

function xOutEmpty : boolean;
begin
	xOutEmpty := (x_status and 16384) > 0;
end;

function xInWaiting : boolean;
begin
	xInWaiting := (x_status and 256) > 0;
end;

function xInEmpty : boolean;
begin
	xInEmpty := (x_status and 256) = 0;
end;


procedure xstrout(s : string);
begin
	x_write(@s[1], length(s));
end;


procedure Hangup;
begin
	if not xCarrier then exit;

	x_dtr(0);
	delay(500);
	x_dtr(1);

	if not xCarrier then exit;

	delay(1000);
	xstrout('+++ATH0'#13);
end;


end.
