unit g_mind;

interface

uses
	 dos, xcrt,
	 slget,
	 defs, mogutils, utils;


function mind_parse(s : string) : boolean;

implementation


const _ver = '1.01';

type  mscore = record
	name 	: string[65];
	moves 	: integer;
	time	: longint;
	th, tm	: word;
	digits	: word;
end;


var
	inited	: boolean;
	target  : string;
	moves	: word;
	ndigits : word;

	sth, stm, sts : word;

procedure start;
var
	n : word;
begin
	if inited then
	begin
		slwriteln('\WH * You are already playing a game!');
		exit;
	end;

	slwriteln('\WH * MogestMind v'+_ver+' initialised.');
	slwriteln('   Type \LC/guess [0-9][0-9][0-9][0-9] \WHto guess.');
	slwriteln('   Type \LC/mind-help \WHfor more help.');

	gettime(sth, stm, sts, n);

	randomize;

	inited := true;

	moves := 0;

	n := random(10000);
	str(n, target);
	while length(target) < ndigits do
		target := '0' + target;
end;



procedure help;
begin
	slwriteln('\YE===== MogestMind Commands\LG');
	slwriteln('  /mogestmind            Start a new game of Mastermind.');
	slwriteln('  /guess [number]        Guess a four digit number.');
	slwriteln('  /mind-top              Display today''s top five Mastermind players.');

	slwriteln(#10'\WHHow do I play MogestMind?');
	slwrite('\CYType \LC/mogestmind \CYto start.  The computer will choose a four digit number '+
			'between 0000 and 9999 inclusive.  Your goal is to find that number in the minimum '+
			'number of moves and the least time.'+#13#10#10+'You can guess the number by typing \lc/guess ');

	slwrite('1234\cy, where 1234 is the number you want to guess.  The computer will respond '+
			'with the number of digits you got right in the right place, and the number of digits '+
			'you got right but not in the right place.'+#13#10#10+'Note the computer can choose a number ');

	slwriteln('with digits occuring more than once (for example 4221).  This can be confusing!  '+
			  'Experiment with the response system to try to see how it determines what a right '+
			  'place is.  Try asking people on-line MogTalk now for some strategy hints!');


end;

procedure top;
var
	fptr : file;
	l    : longint;
	td	 : DateTime;
	a, b : word;

	ms   : array [1..5] of mscore;
begin
	filemode := $40+$2;
	assign(fptr, 'mind.top');
	{$i-} reset(fptr, 1); {$I+}
	if ioresult <> 0 then
	begin
		slwriteln('\WH * No one has played MogestMind today!  Please play!');
		exit;
	end;

	getftime(fptr, l);
	unpacktime(l, td);

	getdate(a, a, b, a);

	if b <> td.day then
	begin
		close(fptr);
		erase(fptr);
		slwriteln('\WH * No one has played MogestMind today!  Please play!');
		exit;
	end;

	slwriteln('\YE === Top Five MogestMind Players for Today ===\LG');

	blockread(fptr, ms, sizeof(mscore)*5);
	close(fptr);

	for a := 1 to 5 do
	begin
		with ms[a] do if name <> '' then
		begin
			slwrite('  '+name+sp(25-length(stripcol(name)))+'\LGtook \WH'+
					strs(moves)+'\LG move'+ps(moves)+' and \WH');

			if time > 300 then
				slwrite(strs(time div 60)+' \LGminute'+ps(time div 60)+' ')
			else
				slwrite(strs(time)+' \LGsecond'+ps(time)+' ');

			slwriteln('at \WH'+strs2(th)+':'+strs2(tm)+'\LG.');
		end;
	end;
end;

function time_elapsed(h, m, s : word) : longint;
var
	a, b, c, d : word;
	l1, l2	   : longint;
begin
	gettime(a, b, c, d);

	l1 := a * 60 * 60 + b * 60 + c;
	l2 := h * 60 * 60 + m * 60 + s;

	if l1 < l2 then inc(l1, 86400);

	time_elapsed := l1 - l2;
end;


procedure add_name_maybe;
var
	fptr : file;
	l    : longint;
	td	 : DateTime;
	a, b, c, n : word;
	lp	 : integer;
	ms   : array [1..5] of mscore;
	te	 : longint;


  procedure writeit;
  var
	lp, pp, t : word;
	temp	  : mscore;
  begin
	for lp := 1 to 4 do
	begin
		t := lp;

		for pp := lp+1 to 5 do
			if ms[pp].time < ms[t].time then
				t := pp;

		if t <> lp then
		begin
			move(ms[lp], temp, sizeof(mscore));
			move(ms[t], ms[lp], sizeof(mscore));
			move(temp, ms[t], sizeof(mscore));
		end;
	end;

	for lp := 1 to 4 do
	begin
		t := lp;

		for pp := lp+1 to 5 do
			if ms[pp].moves < ms[t].moves then
				t := pp;

		if t <> lp then
		begin
			move(ms[lp], temp, sizeof(mscore));
			move(ms[t], ms[lp], sizeof(mscore));
			move(temp, ms[t], sizeof(mscore));
		end;
	end;

	seek(fptr, 0);
	blockwrite(fptr, ms, sizeof(mscore)*5);
	close(fptr);
  end;

label blank;

begin
	te := time_elapsed(sth, stm, sts);

	filemode := $40+$2;
	assign(fptr, 'mind.top');
	{$i-} reset(fptr, 1); {$I+}
	if ioresult <> 0 then
	begin
blank:  fillchar(ms, sizeof(mscore)*5, 0);
		ms[1].name := myrec.alias;
		ms[1].moves := moves;
		ms[1].time := te;
		ms[1].digits := ndigits;
		gettime(ms[1].th, ms[1].tm, n, n);
		rewrite(fptr, 1);
		writeit;
		exit;
	end;

	getftime(fptr, l);
	unpacktime(l, td);

	getdate(a, a, b, a);

	if b <> td.day then
	begin
		close(fptr);
		erase(fptr);
		goto blank;
	end;

	a := 0;
	b := 0;
	c := 0;

	blockread(fptr, ms, sizeof(mscore)*5);
	for lp := 1 to 5 do
	begin
		if ms[lp].name = '' then
		begin
			ms[lp].name := myrec.alias;
			ms[lp].moves := moves;
			ms[lp].time := te;
			ms[lp].digits := ndigits;
			gettime(ms[lp].th, ms[lp].tm, n, n);
			writeit;
			exit;
		end;

		if (ms[lp].moves > a) or ((ms[lp].moves = a) and (ms[lp].time > c)) then
		begin
			a := ms[lp].moves;
			c := ms[lp].time;
			b := lp;
		end;
	end;

	if (moves < a) or ((moves = a) and (te < c)) then
	begin
		ms[b].name := myrec.alias;
		ms[b].moves := moves;
		ms[b].time := time_elapsed(sth, stm, sts);
		ms[b].digits := ndigits;
		gettime(ms[b].th, ms[b].tm, n, n);
		writeit;
	end;
end;

procedure guess(s : string);
var
	lp 			: integer;
	rnum, rpos 	: integer;
begin
	if inited = false then
	begin
		slwriteln('\WH * Type /mogestmind to start MogestMind.');
		exit;
	end;

	if length(s) <> 4 then
	begin
		slwrite('\WH * Your guess must consist of '+strs(ndigits)+' numbers (eg /guess ');
		for lp := 1 to ndigits do
			xwrite(strs(random(10)));
		xwriteln(')');
		exit;
	end;

	rnum := 0;
	rpos := 0;

	for lp := 1 to 4 do
	begin
		if not (s[lp] in ['0'..'9']) then
		begin
			slwrite('\WH * Your guess must consist of '+strs(ndigits)+' numbers (eg /guess ');
			for lp := 1 to ndigits do
				xwrite(strs(random(10)));
			xwriteln(')');
			exit;
		end;

		if s[lp] = target[lp] then
			inc(rpos)
		else if pos(s[lp], target) > 0 then
			inc(rnum);
	end;

	inc(moves);

	if rpos = ndigits then
	begin
		slwriteln('\WH * Congratulations!  You guessed it in '+strs(moves)+' moves!');
		add_name_maybe;
		inited := false;
		exit;
	end;

	slwriteln('\WH * Guess '+s+': '+strs(rpos)+' correct and right place, '+strs(rnum)+' correct but wrong place.');
	exit;
end;


function mind_parse(s : string) : boolean;
var
	w1 : string;
begin
	mind_parse := true;

	w1 := aword(s, 1, false);

	if s = '/mogestmind' then
	begin
		enteruserline;
		start;
		exit;
	end;

	if w1 = '/mind-help' then
	begin
		enteruserline;
		help;
		exit;
	end;

	if w1 = '/mind-top' then
	begin
		enteruserline;
		top;
		exit;
	end;


	if inited = false then
	begin
		mind_parse := false;
		exit;
	end;

	if w1 = '/guess' then
	begin
		enteruserline;
		guess(aword(s, 2, false));
		exit;
	end;

	mind_parse := false;
end;



begin
	inited := false;
	ndigits := 4;
end.