unit mogfile;

interface

uses mogutils, defs, utils, xcrt, dos, general, crt;


type gutype = ( Okay, NoMatchFound, NotAccurate );
type swtype = ( CurrentChannel, AllChannels );


procedure OpenSharedFile(s : string; filem : word);
procedure OpenSharedFileF(var sf : sfhtype; s : string; filem : word);
procedure CloseSharedFile;
procedure CloseSharedFileF(var sf : sfhtype);

function GetUserRec(s : string; SearchWhat : swtype; perror : boolean; var vr : sharedrec) : gutype;
function GetUserNode(ss : string; SearchAllChannels : boolean) : word;
function GetUserAlias(ss : string) : string;
function GetUserAbbr(ss : string) : string;
function GetCUserRec(un : string; var pok : cusertype) : boolean;
function GetAName(s : string) : string;

procedure ReadChannelInfo;
procedure WriteCInfo;
procedure ReadCInfo;


implementation

type
{ Typed and untyped files }
  TFileRec = record
    Handle: Word;
    Mode: Word;
    RecSize: Word;
    Private: array[1..26] of Byte;
    UserData: array[1..16] of Byte;
    Name: array[0..79] of Char;
  end;


procedure OpenSharedFile(s : string; filem : word);
begin
	OpenSharedFileF(sf, s, filem);
end;

function open(var f : file; fname : string; openmode : word) : boolean;
var
  regs		: registers;
  a			: ^TFileRec;
label reopen;
begin
	a := @f;

reopen:
	{ Win95 has dos versions between 7 and 20,
      OS/2 has dos versions 20 and above }

	if (lo(dosversion) >= 7) and (lo(dosversion) < 20) then
	begin
		regs.ax := $716c;
		regs.bx := openmode;{ or 256 or 16384;}
	end
	else
	begin
		regs.ax := $6c00;
		regs.bx := openmode;
	end;

	regs.cx := 0;
	regs.dx := $11; 	{ open and create if it doesn't exist }

	assign(f, fname);

	regs.ds := seg(a^.name);
	regs.si := ofs(a^.name);

	intr($21, regs);

	if regs.flags and 1 = 0 then
	begin
		a^.handle := regs.ax;
		a^.recsize := 1;
		a^.mode := 55219;
		open := true;
	end
	else
		open := false;
end;


procedure OpenSharedFileF(var sf : sfhtype; s : string; filem : word);
var
	quit 		: boolean;
	n 			: integer;
	oldshare	: byte;
	lc			: longint;
	a			: ^TFileRec;
begin
	lc := 0;

	if sf.isopen = true then exit;

	oldshare := filemode;

	assign(sf.f, s);
	filemode := filem + FileMode95;

	a := @sf.f;

	quit := false;

	repeat
		inc(lc);

		quit := open(sf.f, s, filem);

		if (lc > 5) and (xCarrierDetect = false) then
		begin
			slwriteln('\WH * Lost carrier while trying to open file.');
			halt;
		end;

	until quit = true;

	filemode := oldshare;
	sf.isopen := true;
end;


procedure CloseSharedFile;
begin
	CloseSharedFileF(sf);
end;

procedure CloseSharedFileF(var sf : sfhtype);
begin
	if (sf.isopen = true) then
	begin
		close(sf.f);
		sf.isopen := false;
	end;
end;


function GetUserRecCur(s : string; perror : boolean; var vr : sharedrec) : gutype;
var
	ExactMatch 	: boolean;
	Matches		: integer;
	LastRec		: sharedrec;
	Quit		: boolean;

	lp, n		: integer;
	ss, st, su	: string;
	r			: sharedrec;
begin
	ss := stripcol(ups(s));

	ExactMatch := false;
	Matches := 0;

	for lp := 1 to topnode do
	begin
		if (ccnodes[lp] <> Nil) and not ccnodes[lp]^.invisible then
		begin
			st := ccnodes[lp]^.alias;
			st := stripcol(ups(st));
			if (ss = copy(st, 1, length(ss))) and (ExactMatch = false) then
			begin
				if ss = st then
					ExactMatch := true;

				inc(Matches);
				move(ccnodes[lp]^, LastRec, sizeof(r));
			end;
		end;
	end;

	if Matches = 0 then
	begin
		for lp := 1 to topnode do
		begin
			if ccnodes[lp] <> Nil then
			begin
				st := ccnodes[lp]^.alias;
				st := stripcol(ups(st));

				n := 1;
				quit := false;
				su := ' ';

				while (not quit) and (su <> '') and (n <= length(ss)) do
				begin
					su := aword(st, n, false);
					if su <> '' then
					begin
						if ss[n] <> su[1] then
							quit := true;

						inc(n);
					end;
				end;

				if (quit = false) and (length(ss) <= n-1) then
				begin
					inc(Matches);
					move(ccnodes[lp]^, LastRec, sizeof(r));
				end;
			end;
		end;

		if Matches = 0 then
		begin
			if perror then
				slwrite('\WH * User \LG'+s+'\WH is not on this channel!'#13#10);

			GetUserRecCur := NoMatchFound;
			exit;
		end;
	end;

	if (ExactMatch = false) and (Matches > 1) then
	begin
		if perror then
			slwrite('\WH * The name you specified is not specific enough.'#13#10);

		GetUserRecCur := NotAccurate;
		exit;
	end;

	move(LastRec, vr, sizeof(vr));

	GetUserRecCur := Okay;
end;



function GetUserRec(s : string; SearchWhat : swtype; perror : boolean; var vr : sharedrec) : gutype;
var
	ExactMatch 	: boolean;
	Matches		: integer;
	LastRec		: sharedrec;

	ss, st		: string;
	r			: sharedrec;

begin
	ss := stripcol(ups(s));
	stripspaces(ss);

	if ss = '' then
	begin
		if perror then
			slwrite('\WH * The name you specified is not specific enough.'#13#10);

		GetUserRec := NotAccurate;
		exit;
	end;

	if SearchWhat = CurrentChannel then
	begin
		GetUserRec := GetUserRecCur(s, perror, vr);
		exit;
	end;

	ExactMatch := false;
	Matches := 0;

	seek(usf.f, 0);
	while not eof(usf.f) do
	begin
		blockread(usf.f, r, sizeof(r));
		if (r.node > 0) and not r.invisible then
		begin
			st := r.alias;
			st := stripcol(ups(st));
			if (ss = copy(st, 1, length(ss))) and (ExactMatch = false) then
			begin
				if ss = st then
					ExactMatch := true;

				inc(Matches);
				move(r, LastRec, sizeof(r));
			end;
		end;
	end;

	if Matches = 0 then
	begin
		if perror then
			slwrite('\WH * User \LG'+s+'\WH is not on this channel!'#13#10);

		GetUserRec := NoMatchFound;
		exit;
	end;

	if (ExactMatch = false) and (Matches > 1) then
	begin
		if perror then
			slwrite('\WH * The name you specified is not specific enough.'#13#10);

		GetUserRec := NotAccurate;
		exit;
	end;

	move(LastRec, vr, sizeof(vr));

	GetUserRec := Okay;
end;



function GetUserNode(ss : string; SearchAllChannels : boolean) : word;
var
	st : swtype;
	r : sharedrec;
begin
	GetUserNode := 0;

	if SearchAllChannels = true then
		st := AllChannels
	else
		st := CurrentChannel;

	if GetUserRec(ss, st, false, r) = Okay then
		GetUserNode := r.node;
end;

function GetUserAlias(ss : string) : string;
var
	r : sharedrec;
begin
	GetUserAlias := '';

	if GetUserRec(ss, CurrentChannel, false, r) = Okay then
		GetUserAlias := r.alias;
end;

function GetUserAbbr(ss : string) : string;
var
	r : sharedrec;
begin
	GetUserAbbr := '';

	if GetUserRec(ss, CurrentChannel, false, r) = Okay then
		GetUserAbbr := r.abbrname;
end;


function GetCUserRec(un : string; var pok : cusertype) : boolean;
var
	au : cusertype;
	s  : string;
	sp : longint;
	ignoreotherstuff : boolean;
begin
	getcuserrec := false;
	ignoreotherstuff := false;

	un := stripcol(ups(un));

	sp := 0;
	OpenSharedFile(cfg.channeldir+'MOGTALK.USR', IRW+DenyNone);
	while not eof(sf.f) do
	begin
		{$i-} blockread(sf.f, au, sizeof(au)); {$I+}
		if ioresult <> 0 then
		begin
			slwriteln('\WH * Disk read error :-(');
			CloseSharedFile;
			exit;
		end;

		s := stripcol(ups(au.alias));

		if (un = copy(s, 1, length(un))) and (ignoreotherstuff = false) then
		begin
			if un = s then
			begin
				ignoreotherstuff := true;
				sp := 0;
			end;

			if sp <> 0 then
			begin
				slwrite('\WH * The name you specified was not specific enough.'#13#10);
				CloseSharedFile;
				exit;
			end;

			sp := filepos(sf.f);
			move(au, pok, sizeof(cusertype));
		end;
	end;

	if sp = 0 then
	begin
		slwrite('\WH * The name you specified was not found in the userbase.'#13#10);
		CloseSharedFile;
		exit;
	end;

	dec(sp, sizeof(au));
	seek(sf.f, sp);

	getcuserrec := true;
end;


function GetAName(s : string) : string;
var
  ct	: cusertype;

begin
	s := ups(s);

	OpenSharedFile(cfg.channeldir+'MOGTALK.USR', IRW+DenyNone);
	while not eof(sf.f) do
	begin
		{$i-} blockread(sf.f, ct, sizeof(ct)); {$I+}
		if ioresult <> 0 then
		begin
			slwriteln('\WH * Disk read error :-(');
			CloseSharedFile;
			exit;
		end;

		if ct.name = s then
		begin
			CloseSharedFile;
			GetAName := ct.alias;
			exit;
		end;
	end;
	CloseSharedFile;

	GetAName := '';
end;

procedure WriteCInfo;
var
	chsf : sfhtype;
begin
	chsf.isopen := false;
	OpenSharedFileF(chsf, cfg.channeldir+strs(myrec.channel)+'.CHA', IWrite+DenyNone);
	blockwrite(chsf.f, mychan, sizeof(mychan));
	CloseSharedFileF(chsf);
end;


procedure ReadCInfo;
var
	chsf : sfhtype;
begin
	chsf.isopen := false;
	OpenSharedFileF(chsf, cfg.channeldir+strs(myrec.channel)+'.CHA', IRW+DenyNone);

	if filesize(chsf.f) = 0 then
	begin
		fillchar(mychan, sizeof(mychan), 0);
		seek(chsf.f, 0);
		blockwrite(chsf.f, mychan, sizeof(mychan));
	end
	else
		blockread(chsf.f, mychan, sizeof(mychan));

	CloseSharedFileF(chsf);
end;


procedure ReadChannelInfo;
var
	rec		: sharedrec;
	n		: longint;
	lp		: integer;

begin
	if not usf.isopen then exit;

	ReadCInfo;

	for lp := 1 to topnode do
	begin
		if ccnodes[lp] <> Nil then
		begin
			freemem(ccnodes[lp], sizeof(sharedrec));
			ccnodes[lp] := Nil;
		end;
	end;

	n  := 0;
	lp := 1;
	topnode := 1;

	seek(usf.f, 0);

	while not eof(usf.f) do
	begin
		blockread(usf.f, rec, sizeof(rec));

		if (rec.node > 0) and (rec.channel = myrec.channel) then
		begin
			if ccnodes[rec.node] = Nil then
				getmem(ccnodes[rec.node], sizeof(sharedrec));

			move(rec, ccnodes[rec.node]^, sizeof(sharedrec));

			if topnode < rec.node then
				topnode := rec.node;
		end;

		inc(n, sizeof(rec));
		inc(lp);
	end;
end;



end.
