unit oth_gen;

interface


uses	oth_def,
		xcrt, dos,
		mogutils, defs, mogfile, utils;

procedure SaveScr;
procedure ResetScr;

procedure MakePos(x, y : integer; var p : position);
function  GetP(game : gamet; p : position) : piece;
procedure PutP(var game : gamet; p : position; pi : piece);
function  GetColor(game : gamet; p : position) : piece;
function  InvColor(c : piece) : piece;
procedure WritePiece(p : position; pi : piece);
function  PosTranslate(s : string; var p : position) : boolean;
function  GetStrength(game : gamet; col : piece) : integer;
procedure DisplayScreen(game : gamet);

function  GetGame(n : integer; var game : gamet) : boolean;
procedure PutGame(n : integer; game : gamet);
function MakeGame(me, them : string) : word;


implementation

function MakeGame(me, them : string) : word;
var
	n : integer;
	fptr : file;
	x : integer;
	game : gamet;
begin
	fillchar(game, sizeof(game), 0);
	with game do
	begin
		player_w := me;
		player_b := them;
		turn := PWhite;
		state := InProgress;
		board[4,4] := PWhite;
		board[5,5] := PWhite;
		board[4,5] := PBlack;
		board[5,4] := PBlack;
	end;

	filemode := DenyNone + $2;
	n := 0;

	repeat
		inc(n);
		assign(fptr, strs(n)+'.OTH');
		{$i-} reset(fptr, 1); {$I+}
		x := ioresult;
		if x = 0 then close(fptr);
	until x <> 0;

	OpenSharedFile(cfg.gamedir+strs(n)+'.OTH', IWrite+DenyAll);
	blockwrite(sf.f, game, sizeof(gamet));
	CloseSharedFile;

	MakeGame := n;
end;




function GetGame(n : integer; var game : gamet) : boolean;
begin
	GetGame := false;
	if n < 1 then exit;

	OpenSharedFile(cfg.gamedir+strs(n)+'.OTH', IRead+DenyNone);

	if filesize(sf.f) = 0 then
	begin
		CloseSharedFile;
		erase(sf.f);
		exit;
	end;

	blockread(sf.f, game, sizeof(gamet));
	CloseSharedFile;

	GetGame := true;
end;

procedure PutGame(n : integer; game : gamet);
begin
	OpenSharedFile(cfg.gamedir+strs(n)+'.OTH', IWrite+DenyAll);
	blockwrite(sf.f, game, sizeof(gamet));
	CloseSharedFile;
end;


procedure SaveScr;
begin
	sx := xwherex; sy := xwherey;
	if sy < 15 then sy := 15;
	xwindow(1, 1, scrbx, scrby);
end;

procedure ResetScr;
begin
	xwindow(1, 15, scrbx, scrby);
	xgotoxy(sx, sy);
end;


procedure MakePos(x, y : integer; var p : position);
begin
	p.x := x;
	p.y := y;
end;


function GetP(game : gamet; p : position) : piece;
begin
	GetP := game.board[p.y, p.x];
end;

procedure PutP(var game : gamet; p : position; pi : piece);
begin
	game.board[p.y, p.x] := pi;
end;

function GetColor(game : gamet; p : position) : piece;
var
	pi : piece;
begin
	pi := GetP(game, p);

	GetColor := pi;
end;

function InvColor(c : piece) : piece;
begin
	case c of
	  PWhite: InvColor := PBlack;
	  PBlack: InvColor := PWhite;
	  PNone:  InvColor := PNone;
	end;
end;

procedure WritePiece(p : position; pi : piece);
begin
	xgotoxy(29+(p.x*3-3), 3+(9-p.y));

	if (p.x mod 2)+(p.y mod 2) = 1 then
	begin
		xtextbackground(white);
		slwrite(PieceName[pi]);
		xtextbackground(black);
	end
	else
		slwrite(PieceName[pi]);
end;

function PosTranslate(s : string; var p : position) : boolean;
begin
	s := ups(s);

	PosTranslate := false;

	if length(s) < 2 then exit;

	if not (s[1] in ['A'..'H']) then
		exit;

	if not (s[2] in ['0'..'8']) then
		exit;

	p.x := ord(s[1]) - 64;
	p.y := ord(s[2]) - 48;

	PosTranslate := true;
end;

function GetStrength(game : gamet; col : piece) : integer;
var
	lp, pp, n : integer;
	p		  : position;
begin
	n := 0;

	for lp := 1 to 8 do
	begin
		p.y := lp;
		for pp := 1 to 8 do
		begin
			p.x := pp;

			if GetColor(game, p) = col then
				inc(n);
		end;
	end;

	GetStrength := n;
end;

procedure DisplayScreen(game : gamet);
var
	lp, pp	: integer;
	p		: position;

begin
	for lp := 1 to 8 do
	begin
		for pp := 1 to 8 do
		begin
			p.x := pp;
			p.y := lp;
			WritePiece(p, GetP(game, p));
		end;
	end;
end;


end.
