unit slmem;

interface

uses	dos, menudef;


const MaxStack = 40;    { menu stack maximum size }
	  SLBBSID = $736C;  { code for interrupt identifications }
      RSInt = $7E;      { modem driver interrupt }

type
	 Ansitype = (GENERIC,PROCOMM,STANDARD);

     { Following is the data format in which the menu stack is stored
	   in memory during door execution. With care, door programs can
       modify this data area in order to produce changes in menu
       execution when control returns from the door to Searchlight. }

     MenuData = Record
	   Count: integer;          { # of active menus }
	   FilesCount: integer;     { menu that called files system, 0 if none }
	   Menus: Array[1..MaxStack] of record
         Name: string[8];       { menu filename }
		 Pos: integer;          { cursor position within menu }
         Mode: ExecType;        { display/execute }
	   end;
	 end;

Type SlDataType = record   { public area; contains menu stack }
      PROGID: string[6];                { Program ID }
      carrier: boolean;                 { carrier check enable }
      writeprotect: boolean;            { disk write protection }
      aborttype: byte;                  { 0=no abort, 1=terminate, 2=reboot }
      rsactive: boolean;                { set if rs232 active }
      ansi: boolean;                    { user ANSI mode }
      color: boolean;                   { user COLOR mode }
      directvid: boolean;               { system DirectVid mode }
      curratt: byte;                    { current video attribute }
      commtype: byte;                   { run parameter }
	  idletime: word;                   { idle limit (seconds) }
	  lastkey: boolean;                 { TRUE = last key from local kbd }
      OldVector: array[$00..$7F] of pointer;   { old user int vectors }
      AnsiMode: AnsiType;               { caller's ansi mode }
      SLMenuStack: MenuData;            { active menu stack }
    end;

var
	SLData			: ^SLDataType;


procedure AddMenuToStack(s : string; de : exectype);
procedure RemoveMenuFromStack(num : integer);


implementation


procedure AddMenuToStack(s : string; de : exectype);
var
	n : integer;
begin
	inc(SLData^.SLMenuStack.Count);
	n := SLData^.SLMenuStack.Count;

	SLData^.SLMenuStack.Menus[n].Name := s;
	SLData^.SLMenuStack.Menus[n].Pos := 1;
	SLData^.SLMenuStack.Menus[n].Mode := de;
end;

procedure RemoveMenuFromStack(num : integer);
var
	n : ^integer;
begin
	n := @SLData^.SLMenuStack.Count;

	if n^-num < 1 then
		n^ := 0
	else
		n^ := n^-num;
end;



var
	oep, p : pointer;
	regs   : registers;


procedure nep; far;
begin
	exitproc := oep;
end;

begin
	SLData := Nil;

	GetIntVec($79, p);
	if longint(p) = SLBBSID then
	begin
		regs.ax := $C7;
		regs.cx := $00;
		intr(RSInt, regs);
		if regs.cx = SLBBSID then
			SLData := Ptr(regs.ax, regs.bx);
	end;

	if SLData = Nil then
	begin
		new(SLData);
		fillchar(SLData^, sizeof(SLDataType), 0);
		SLData^.PROGID[0] := chr($6c);
		SLData^.PROGID[1] := chr($73);
	end;

	oep := exitproc;
	exitproc := @nep;
end.
