unit xcrt;

interface

{$define usesl}

{$ifdef usesl}
uses 	crt, modem, slfossil, multi, utils;
{$else}
uses 	crt, fossil, multi, utils, defs;
{$endif}


procedure xwindow(tx, ty, bx, by : word);
procedure xdelay(w : word);
procedure xstrwrite(s : string);

procedure xinsline;

procedure xwrite(s : string);
procedure xwriteln(s : string);

procedure extextcolor(w, hi : word);
procedure xtextcolor(w : word);
procedure xtextbackground(w : word);
function  xwherey : word;
function  xwherex : word;

procedure xgotoxy(x, y : word);
procedure xclreol;

function  xreadkey : char;
function  xkeypressed : boolean;

procedure xclrscr;

procedure xhangup;

function  xCarrierDetect : boolean;

function  xtextattr : word;

const
	Black 		 = 0;
	Blue  		 = 1;
	Green 		 = 2;
	Cyan  		 = 3;
	Red   		 = 4;
	Magenta   	 = 5;
	Brown     	 = 6;
	LightGray 	 = 7;
	DarkGray  	 = 8;
	LightBlue 	 = 9;
	LightGreen   = 10;
	LightCyan    = 11;
	LightRed   	 = 12;
	LightMagenta = 13;
	Yellow       = 14;
	White        = 15;
	Blink		 = 128;

	scrby		 = 24;

type
	vsptype		= record
		x, y		: word;
	end;


var
	local		: boolean;
	scrty0		: word;

	readbuf		: string;

	vsp			 : vsptype;
	ScreenLength : integer;

implementation

procedure xwindow(tx, ty, bx, by : word);
begin
	window(tx, ty, bx, by);
	scrty0 := ty - 1;
	vsp.x := 1;
	vsp.y := ty;
end;

procedure xstrwrite(s : string);
begin
	if not local then
		x_write(@s[1], length(s));
end;

procedure xdelay(w : word);
begin
	delay(w);
end;

procedure xhangup;
begin
	hangup;
end;

procedure xinsline;
begin
	insline;
end;


function xreadkey : char;
var
	ch : char;
begin
	if readbuf <> '' then
	begin
		xreadkey := readbuf[1];
		delete(readbuf, 1, 1);
		exit;
	end;

	if local then
	begin
		while (not keypressed) and (XCarrierDetect) do Slice;


		if XCarrierDetect = false then
		begin
			xreadkey := #255;
			exit;
		end;

		xreadkey := readkey;
		exit;
	end;

	while true do
	begin
		if keypressed then
		begin
			xreadkey := readkey;
			exit;
		end;

		if xInWaiting then
		begin
			ch := char(x_getc);
			if (ch <> #27) or (not xInWaiting) then
			begin
				xreadkey := ch;
				exit;
			end;

			pause(4);

			ch := char(x_getc);
			if ch <> '[' then
			begin
				readbuf := readbuf + ch;
				xreadkey := #27;
				exit;
			end;

			ch := char(x_getc);
			xreadkey := #0;

			case ch of
			  'C':	readbuf := readbuf + #77;
			  'D':  readbuf := readbuf + #75;
			  'H':  readbuf := readbuf + #71;
			  'K':  readbuf := readbuf + #79;
			  'A':  readbuf := readbuf + #72;
			  'B':  readbuf := readbuf + #80;
			else
				xreadkey := ch;
			end;

			exit;
		end;

		if xCarrierDetect = false then
		begin
			xreadkey := #255;
			exit;
		end;

		Slice; Slice;
	end;
end;

function xwherex : word;
begin
	xwherex := vsp.x;
end;

function xwherey : word;
begin
	xwherey := vsp.y;
end;


function xkeypressed : boolean;
begin
	if local then
		xkeypressed := keypressed
	else
		xkeypressed := keypressed or xInWaiting;
end;

procedure xgotoxy(x, y : word);
begin
	if y > scrby then
	begin
		if vsp.y = y then
			gotoxy(x, wherey-scrty0)
		else if vsp.y > y then
			gotoxy(x, wherey-(vsp.y-y)-scrty0)
		else
			gotoxy(x, scrby-scrty0)
	end
	else
		gotoxy(x, y-scrty0-(vsp.y-(wherey+scrty0)));

	vsp.x := x;
	vsp.y := y;

	if not local then
	begin
		if x = 1 then
			xstrwrite(#27'['+strs(y)+'H')
		else
			xstrwrite(#27'['+strs(y)+';'+strs(x)+'H');
	end;
end;

procedure xclreol;
begin
	clreol;
	if not local then
		xstrwrite(#27'[K');
end;

var
	lastcolor : word;



procedure extextcolor(w, hi : word);
var
	p, n : integer;
	s : string;
begin
	textcolor(w);
	if not local then
	begin
		p := w and 7;
		s := 'argh';
		case p of
			black:		s := '30';
			red:    	s := '31';
			green:		s := '32';
			brown: 		s := '33';
			blue:		s := '34';
			magenta:	s := '35';
			cyan:		s := '36';
			lightgray:	s := '37';
		end;

		if ((w and 127 > 7) and (lastcolor and 127 < 8)) or
		   ((lastcolor and 128 = 128) and (w and 128 = 0) and (lastcolor and 127 > 7)) then
			s := '1;'+s;

		if ((w and 127 < 8) and (lastcolor and 127 > 7)) or
		   ((lastcolor and 128 = 128) and (w and 128 = 0)) then
		begin
			s := '0;'+s
		end;

		if w and 128 = 128 then
			s := s + ';5';

		if hi <> 0 then
			s := strs(hi+256) + ';' + s;

		xstrwrite(#27'['+s+'m');

		lastcolor := w;
	end;
end;

procedure xtextcolor(w : word);
begin
	extextcolor(w, 0);
end;


procedure xtextbackground(w : word);
var
	p, n : integer;
begin
	textbackground(w);
	if not local then
	begin
		p := w and 7;
		case p of
			black:		n := 40;
			red:    	n := 41;
			green:		n := 42;
			brown: 		n := 43;
			blue:		n := 44;
			magenta:	n := 45;
			cyan:		n := 46;
			lightgray:	n := 47;
		end;

		xstrwrite(#27'['+strs(n)+'m');
	end;

end;

procedure xclrscr;
begin
	clrscr;
	if not local then
		xstrwrite(#27'[2J');
	vsp.x := 1;
	vsp.y := scrty0+1;
end;

procedure xwrite(s : string);
var
	lp : integer;
begin
	for lp := 1 to length(s) do
	begin
		case s[lp] of
		  #8:
		  begin
			if (vsp.y > 1) or (vsp.y > 1) then
			begin
				dec(vsp.x);
				if vsp.x = 0 then
				begin
					vsp.x := 80;
					dec(vsp.y);
				end;
			end;
		  end;

		  #10:
		  begin
			if vsp.y < ScreenLength then
				inc(vsp.y);
		  end;

		  #13:	vsp.x := 1;

		  #32..#255:
		  begin
			inc(vsp.x);
			if vsp.x = 81 then
			begin
				vsp.x := 1;
				if vsp.y < ScreenLength then
					inc(vsp.y);
			end;
		  end;
		end;
	end;

	write(s);

	if not local then
		xstrwrite(s);
end;

procedure xwriteln(s : string);
begin
	xwrite(s+#13#10);
end;

function xCarrierDetect : boolean;
begin
{$ifdef usesl}
	xCarrierDetect := CarrierDetect;
{$else}
	if local then
		xCarrierDetect := true
	else
		xCarrierDetect := xCarrier;
{$endif}
end;

function xtextattr : word;
begin
	xtextattr := textattr;
end;

begin
{$ifdef usesl}
	local := (not SLData^.rsact) or (SLData = Nil);
{$else}
	local := true;
{$endif}

	checkbreak := false;
	readbuf := '';
	scrty0 := 0;
	ScreenLength := 24;
	lastcolor := cyan;
	vsp.x := wherex;
	vsp.y := wherey;
	xstrwrite(#27'[0;36m');
end.
